#include "QueensTest.hpp"
#include "stdlib.h"
#include "stdio.h"
#include <time.h>
#include <sys/timeb.h>


long 
QueensTest::doQueenAllocationFrom( int queenCol, Board& board )
/////////////////////////////////////////////////////////////////////
{
  const int N = board.getSideSize();

  long nSolutions = 0;
  for ( int row = 0; row < N; row++ )
  {
    if ( !board.isAttacked( row, queenCol ) )
    {
      board.setQueenRow( queenCol, row );
      if ( queenCol == N - 1 ) // recursion base
        nSolutions++;
      else // recursion
        nSolutions += doQueenAllocationFrom( queenCol + 1, board );
    } // if
  } // for row

  // Clean value setted in setQueenRow, if necessary.
  board.cleanQueenCol( queenCol );

  return nSolutions;
}


void 
QueensTest::doIndividualTest( int sideSize )
////////////////////////////////////////////
{
  struct timeb start, finish;
  Board board( sideSize );

  ftime( &start );
  long nSolutions   = doQueenAllocationFrom( 0, board );
  ftime( &finish );

  long duration = (long)( finish.time * 1000 + finish.millitm - 
                          (start.time * 1000 + start.millitm) );

  if ( nSolutions > 0 )
    printf( "Side=%d, [%d ms]: %d solutions\n", sideSize, duration, 
            nSolutions );
  else
    printf( "Side=%d, [%d ms]: couldn't be solved!\n", sideSize, 
            duration );
}


void 
QueensTest::doIncrementalTest()
///////////////////////////////
{
  for ( int sideSize = 1; true; sideSize++ )
    doIndividualTest( sideSize );
}


void 
QueensTest::run( int argc, char* argv[] )
/////////////////////////////////////////
{
  // Prepare parameters
  int sideSize = 4; // default
  if ( argc > 1 )
  {
    sideSize = atoi( argv[ 1 ] );
  }

  // Run test
  if ( argc > 1 )
    doIndividualTest( sideSize );
  else
    doIncrementalTest();
}
